/****************************************************************************
*
* (C) Unpublished Copyright Novell, Inc. All Rights Reserved.
*
* No part of this file may be duplicated, revised, translated,
* localized or modified in any manner or compiled, linked or 
* uploaded or downloaded to or from any computer system without
* the prior written consent of Novell, Inc.
*
* NetWare C NLM Runtime Library source code
*
*****************************************************************************/

#ifndef _STDIO_H_INCLUDED

#ifndef _SIZE_T_DEFINED_
#define _SIZE_T_DEFINED_
typedef unsigned size_t;
#endif

typedef char    *__va_list[1];
#define NULL 0

#define BUFSIZ    512
#define _NFILES   20 /* number of files that can be handled */
#define FOPEN_MAX _NFILES
#define FILENAME_MAX _MAX_PATH

#pragma pack(1);


#if defined(__WATCOMC__) && defined(__cplusplus)

#if !defined(__NETWARE__)

#error You must use the /bt=netware switch to build a C++ NLM

#else

typedef struct __iobuf {
   unsigned char *_ptr;       /* next character position */
   int            _cnt;       /* number of characters left */
   unsigned char *_base;      /* location of buffer */
   unsigned       _flag;      /* mode of file access */
   int            _handle;    /* file handle */
   unsigned       _bufsize;   /* size of buffer */
   unsigned char  _ungotten;  /* character placed here by ungetc  */
   unsigned char  _tmpfchar;  /* tmpfile number */
} FILE;

extern FILE __iob[];

#define stdin  ((FILE *)&__iob[0]) /* standard input file   */
#define stdout ((FILE *)&__iob[1]) /* standard output file  */
#define stderr ((FILE *)&__iob[2]) /* standard error file   */

#define TMP_MAX (26*26*26)         /*  Max times tmpnam can be called  */

#endif

#else

typedef struct _iobuf {
   char          *_ptr;       /* next character position */
   long           _structid;  /* for file validation */
   int            _cnt;       /* number of characters left */
   char          *_base;      /* location of buffer */
   unsigned       _flag;      /* mode of file access */
   int            _handle;    /* file handle */
   unsigned       _bufsize;   /* size of buffer */
   unsigned char  _ungotten;  /* used as buffer if none allocated */
   unsigned char  _tmpfchar;  /* tmpfile number */
   struct _iobuf *_nextfile;  /* next open file */
   char          *_tmpname;   /* if it is a temporary file */
   int            _semcount;  /* for file locking */
   long           _semaphore; /* ... */
   long           _threadid;  /* ... */
   long           _NCSp;      /* ... */
} FILE;

#ifdef __cplusplus
extern "C" {
#endif

extern FILE **__get_stdin(
         void );

extern FILE **__get_stdout(
         void );

extern FILE **__get_stderr(
         void );

#ifdef __cplusplus
};
#endif

#define stdin  *__get_stdin()
#define stdout *__get_stdout()
#define stderr *__get_stderr()

#define TMP_MAX   32000

#define _SLF_SIGNATURE 0x20464c53       /* signature 'SLF ' */

#endif

typedef long      fpos_t;

/* values for _flag field in FILE struct */

#define _READ   0x0001        /* file opened for reading */
#define _WRITE  0x0002        /* file opened for writing */
#define _UNGET  0x0004        /* ungetc has been done */
#define _BIGBUF 0x0008        /* big buffer allocated */
#define _EOF    0x0010        /* EOF has occurred */
#define _ERR    0x0020        /* error has occurred on this file */
#define _BINARY 0x0040        /* file is binary, skip CRLF processing */
#define _APPEND 0x0080        /* file opened for append */
#define _IOFBF  0x0100        /* full buffering */
#define _IOLBF  0x0200        /* line buffering */
#define _IONBF  0x0400        /* no buffering */
#define _TMPFIL 0x0800        /* this is a temporary file */
#define _DIRTY  0x1000        /* buffer has been modified */
#define _ISTTY  0x2000        /* is console device */
#define _DYNAMIC 0x4000        /* FILE is dynamically allocated */

#define EOF (-1)

#define SEEK_SET  0
#define SEEK_CUR  1
#define SEEK_END  2

#define L_tmpnam  13

#ifndef NO_EXT_KEYS

/* non ANSI specified identifiers and function prototypes are included here */

#define OPEN_MAX FOPEN_MAX
#define PATH_MAX _MAX_PATH

#define fileno(fp)   ((fp)->_handle)

#ifdef __cplusplus
extern "C" {
#endif

extern int fcloseall(
         void);

extern FILE *fdopen(
         int         __handle,
         const char *__mode);

extern int fgetchar(void);

extern int flushall(
         void);

extern int vcprintf(
         const char *__format,
         __va_list __arg);

extern int vcscanf(
         const char *__format,
         __va_list __arg);

extern int vfscanf(
         FILE       *__fp,
         const char *__format,
         __va_list   __arg);

extern int vscanf(
         const char *__format,
         __va_list   __arg);

extern int vsscanf(
         const char *__s,
         const char *__format,
         __va_list   __arg);

#ifdef __cplusplus
}
#endif

#endif

#ifdef __cplusplus
extern "C" {
#endif

extern void clearerr(
         FILE       *__fp);

extern int fclose(
         FILE       *__fp);

extern int feof(
         FILE       *__fp);

extern int ferror(
         FILE       *__fp);

extern int fflush(
         FILE       *__fp);

extern int fgetc(
         FILE       *__fp);

extern int fgetpos(
         FILE       *__fp,
         fpos_t     *__pos);

extern char *fgets(
         char       *__s,
         int         __n,
         FILE       *__fp);

extern FILE *fopen(
         const char *__filename,
         const char *__mode);

extern int fprintf(
         FILE       *__fp,
         const char *__format,
         ...);

extern int fputc(
         int         __c,
         FILE       *__fp);

extern int fputs(
         const char *__s,
         FILE       *__fp);

extern size_t fread(
         void       *__ptr,
         size_t      __size,
         size_t      __n,
         FILE       *__fp);

extern FILE *freopen(
         const char *__filename,
         const char *__mode,
         FILE       *__fp);

extern int fscanf(
         FILE*__fp,
         const char *__format,
         ...);

extern int fseek(
         FILE       *__fp,
         long int    __offset,
         int         __whence);

extern int fsetpos(
         FILE       *__fp,
         const fpos_t *__pos);

extern long int ftell(
         FILE       *__fp);

extern size_t fwrite(
         const void *__ptr,
         size_t      __size,
         size_t      __n,
         FILE       *__fp);

extern int getc(
         FILE       *__fp);

extern int getchar(
         void);

extern char *gets(
         char       *__s);

extern void perror(
         const char *__s);

extern int printf(
         const char *__format,
         ...);

extern int putc(
         int         __c,
         FILE       *__fp);

extern int putchar(
         int         __c);

extern int puts(
         const char *__s);

extern int remove(
         const char *__filename);

extern int rename(
         const char *__old,
         const char *__new);

extern void rewind(
         FILE       *__fp);

extern int scanf(
         const char *__format,
         ...);

extern void setbuf(
         FILE       *__fp,
         char       *__buf);

extern int setvbuf(
         FILE       *__fp,
         char       *__buf,
         int         __mode,
         size_t      __size);

extern int sprintf(
         char       *__s,
         const char *__format,
         ...);

extern int sscanf(
         const char *__s,
         const char *__format,
         ...);

extern FILE *tmpfile(
         void);

extern char *tmpnam(
         char       *__s);

extern int vfprintf(
         FILE       *__fp,
         const char *__format,
         __va_list   __arg);

extern int vprintf(
         const char *__format,
         __va_list   __arg);

extern int vsprintf(
         char       *__s,
         const char *__format,
         __va_list   __arg);

extern int ungetc(
         int         __c,
         FILE       *__fp);

extern FILE **__get_stdin( 
         void );

extern FILE **__get_stdout( 
         void );

extern FILE **__get_stderr( 
         void );

#ifdef __cplusplus
}
#endif

#define clearerr(fp) ((fp)->_flag &= ~(_ERR|_EOF))
#define feof(fp)     ((fp)->_flag & _EOF)
#define ferror(fp)   ((fp)->_flag & _ERR)
#define getchar()    fgetc(stdin)
#define getc(fp)     fgetc((fp))
#define putchar(c)   fputc((c),stdout)
#define putc(c,fp)   fputc((c),(fp))


#pragma pack();

#define _STDIO_H_INCLUDED
#endif
