#ifndef _UNICODE_H_
#define _UNICODE_H_
/****************************************************************************
 * (C) Unpublished Copyright of Novell, Inc.  All Rights Reserved.
 *
 * No part of this file may be duplicated, revised, translated, localized
 * or modified in any manner or compiled, linked or uploaded or downloaded
 * to or from any computer system without the prior written consent of
 * Novell, Inc.
 *
 * This corresponds to the Unicode.NLM
 ****************************************************************************/
#include <stddef.h>

#ifndef TRUE
# define TRUE	1
#endif
#ifndef FALSE
# define FALSE	0
#endif

typedef unsigned short unicode;

#define UNI_FUTURE_OPCODE	-490	/* Rule table has unimplimented rules */
#define UNI_NO_SUCH_FILE	-491	/* No such file or directory */
#define UNI_TOO_MANY_FILES	-492	/* Too many files already open */
#define UNI_NO_PERMISSION	-493	/* Permission denied on file open */
#define UNI_NO_MEMORY		-494	/* Not enough memory */
#define UNI_LOAD_FAILED		-495	/* NWLoadRuleTable failed, don't know why	*/
#define UNI_HANDLE_BAD		-496	/* Rule table handle was bad */
#define UNI_HANDLE_MISMATCH -497	/* Rule table handle doesn't match operation */
#define UNI_RULES_CORRUPT	-498	/* Rule table is corrupt */
#define UNI_NO_DEFAULT		-499	/* No default rule and no 'No map' character */
#define UNI_INSUFFICIENT_BUFFER -500
#define UNI_OPEN_FAILED		-501	/* Open failed in NWLoadRuleTable */
#define UNI_NO_LOAD_DIR		-502	/* Load directory could not be determined */

#define DONT_USE_NOMAP_CHAR	0L		/* for 'noMapFlag' in NWLocalToUnicode() */
#define USE_NOMAP_CHAR			1L		/* and NWUnicodeToLocal() */

#ifdef __cplusplus
	extern "C" {
#endif

int  NWInitUnicodeTables
(
	int	countryCode, 
	int	codePage
);

int  NWLoadRuleTable
(
	char	*ruleTableName,
	void	**ruleHandle
);

int  NWUnloadRuleTable
(
	void	*ruleHandle
);

int  NWLocalToUnicode
(
	void				*ruleHandle,
	unicode			*dest,
	size_t			maxLen,
	char				*src,
	unicode			noMap,
	size_t			*len,
	unsigned long	noMapFlag			/* use definition above */
);

int  NWUnicodeToLocal
(
	void				*ruleHandle,
	char				*dest,
	size_t			maxLen,
	unicode			*src,
	char				noMap,
	size_t			*len,
	unsigned long	noMapFlag			/* use definition above */
);

int  NWUnicodeToCollation
(
	void		*ruleHandle,
	unicode	*dest,
	size_t	maxLen,
	unicode	*src,
	unicode	noMap,
	size_t	*len
);

int  NWUnicodeCompare
(
	void		*ruleHandle,
	unicode	chr1,
	unicode	chr2
);

int  NWUnicodeToMonocase
(
	void		*ruleHandle,
	unicode	*dest,
	size_t	maxLen,
	unicode	*src,
	size_t	*len
);

int  NWFreeUnicodeTables
(
	void
);

int  NWGetUnicodeToLocalHandle
(
	void	**handle
);

int  NWGetLocalToUnicodeHandle
(
	void	**handle
);

int  NWGetMonocaseHandle
(
	void	**handle
);

int  NWGetCollationHandle
(
	void	**handle
);

unicode *unicat
(
	unicode	*s1,
	unicode	*s2
);

unicode *unichr
(
	unicode	*s,
	int		c
);

unicode *unicpy
(
	unicode	*s1,
	unicode	*s2
);

size_t  unilen
(
	unicode	*s
);

unicode *unincat
(
	unicode	*s1,
	unicode	*s2,
	size_t	n
);

unicode *unincpy
(
	unicode	*s1,
	unicode	*s2,
	size_t	n
);

unicode *unipbrk
(
	unicode	*s1,
	unicode	*s2
);

int uniindex
(
	unicode	*source,
	unicode	*searchString
);

int  uniicmp
(
	unicode	*s1,
	unicode	*s2
);

int  uninicmp
(
	unicode	*s1,
	unicode	*s2,
	size_t	n
);

size_t  unisize
(
	unicode	*s
);

unicode *unirev
(
	unicode	*s
);

#ifdef HI_LO_MACH_TYPE
unicode *unicpyswap
(
	unicode	*s1,
	unicode	*s2
);
#endif

#ifdef __cplusplus
	}
#endif

#endif
