/*
//ͻ
// (c) 1993 Novell, Inc.  All rights reserved.                              
//                                                                          
// THIS WORK IS SUBJECT TO U.S. AND INTERNATIONAL COPYRIGHT LAWS AND        
// TREATIES.                                                                
//                                                                          
// NO PART OF THIS WORK MAY BE USED, PRACTICED, PERFORMED, COPIED,          
// DISTRIBUTED, REVISED, MODIFIED, TRANSLATED, ABRIDGED, CONDENSED,         
// EXPANDED, COLLECTED, COMPILED, LINKED, RECAST, TRANSFORMED OR ADAPTED    
// WITHOUT THE PRIOR WRITTEN CONSENT OF NOVELL, INC.                        
//                                                                          
// ANY USE OR EXPLOITATION OF THIS WORK WITHOUT AUTHORIZATION COULD SUBJECT 
// THE PERPETRATOR TO CRIMINAL AND CIVIL LIABILITY.                         
//͹
// Source module name: NWPSPORT.H                                           
//ͼ
*/
#if !defined(NWPSPORT_H)
#define NWPSPORT_H
/*
// This header file contains the portability macros for all supported
// platforms.
*/
#if defined(NWNLM)
 #define N_PLAT_NLM
 #if defined(IAPX386)
 #endif

 #if defined(MC680X0)
  #define        N_INT_ENDIAN_HI_LO
 #endif

 #if defined(MC88000)       /* IBM RISC */
  #define        N_INT_ENDIAN_HI_LO
  #define        N_INT_STRICT_ALIGNMENT
 #endif

 #if defined(RX000)         /* MIPS */
  #define        N_INT_ENDIAN_HI_LO
  #define        N_INT_STRICT_ALIGNMENT
 #endif

 #if defined(SPARC)         /* SUN */
  #define        N_INT_ENDIAN_HI_LO
  #define        N_INT_STRICT_ALIGNMENT
 #endif

 #if defined(HPPA)          /* HP PA-RISC */
  #define        N_INT_ENDIAN_HI_LO
  #define        N_INT_STRICT_ALIGNMENT
 #endif
   
 /*
 // Everything but NLMs is on Intel 386 type platforms.
 */
#elif defined(NWWIN)
  #define N_PLAT_MSW
#elif defined(NWOS2)
  #define N_PLAT_OS2
#elif defined(NWDOS)
  #define N_PLAT_DOS
#endif

#if ! defined( NTYPES_H )
#define NTYPES_H

/*
   Before including this file, the following preprocessor switches may be
   defined.

   One of the following should be defined to specify the platform.  On
   Unix systems, this is derived below from other symbols in native headers,
   but for all other platforms it should be explicitly specified.

   N_PLAT_MSW     Microsoft Windows
   N_PLAT_MAC     Macintosh
   N_PLAT_UNIX    Unix platforms
   N_PLAT_OS2     OS/2
   N_PLAT_DOS     DOS, Character Mode or Graphics Mode

   The following should be defined when a resource compiler is invoked,
   and not defined otherwise:

   N_RC_INVOKED   Defined if a resource compiler is invoked, including
                  Microsoft RC, Apple Rez and the NAWF DRC tool.
*/

/*
   Include native headers that this file depends on, checking to see
   that they weren't previously included whenever possible.
*/

#define NWRCODE long
#define N_PASCAL pascal

#if defined( N_PLAT_MSW )

#ifdef WIN32
#ifndef N_PLAT_WIN32
#define N_PLAT_WIN32
#endif
#else
#ifndef N_PLAT_WIN16
#define N_PLAT_WIN16
#endif
#ifndef NWWIN
#define NWWIN
#endif
#ifndef NWDOS
#define NWDOS
#endif
#endif

   #if ! defined( N_RC_INVOKED )
      #if ! defined( PASCAL )
/*         #include <memory.h> */
         #define WINVER 0x300
         #include <windows.h>
      #endif
   #endif  /* ! defined( N_RC_INVOKED ) */

#elif defined( N_PLAT_MAC )

   #if ! defined( N_RC_INVOKED )
      #if ! defined( __TYPES__ )
         #include <types.h>
      #endif
      #if ! defined( __MEMORY__ )
         #include <memory.h>
      #endif
      #if ! defined( __VALUES__ )
         #include <values.h>
      #endif
      #if defined (THINK_C)
         #include <stdlib.h>
         #include <stdio.h>
         #include <stdarg.h>
         #include <stddef.h>
         #include <string.h>
      #endif
   #endif   /* N_RC_INVOKED */

#elif defined( N_PLAT_UNIX )

   #if !defined(N_RC_INVOKED)
      #include <x11/xos.h>
      #include <values.h>
      #include <sys/types.h>
      #include <memory.h>
      #include <x11/xlib.h>
   #endif

#elif defined( N_PLAT_OS2 )

   #if ! defined( RC_INVOKED )
      #if ! defined( _pascal )
         #include <memory.h>
      #endif
      #if ! defined( APIENTRY )
         #define INCL_PM
         #include <os2.h>
      #endif
   #endif

#endif

/*
------------------------------------------------------------------------------
Constants:

Define the standard constant values
------------------------------------------------------------------------------
*/

/*
   Standard constants.
*/

#if !defined( NULL )
   #define NULL      0
#endif

#define N_NULL       0
#define N_ALWAYS     1
#define N_SUCCESS    0
#define N_FAILURE    (-1)
#define N_YES        1
#define N_NO         0
#define N_FALSE      0
#define N_TRUE       1
#define N_UNKNOWN    (N_TRUE + 1)
#define N_RESTART    (N_TRUE + 2)
#define N_STOP       N_FALSE
#define N_CONTINUE   N_TRUE

/*
   System constants.
*/

#define N_SYS_USER          0
#define N_SYS_NETWARE       1
#define N_SYS_NAWF          2
#define N_SYS_MAX_COUNT     20
#define N_SYS_NAME_MAX_LEN  31

/*
   Historical constant synonyms.
*/

#if ! defined( N_INC_NO_OLD_CONSTANTS )

   #if ! defined( TRUE )
      #define TRUE        N_TRUE
   #endif

   #if ! defined( FALSE )
      #define FALSE       N_FALSE
   #endif

#endif

/* _N_BITSPERBYTE is currently only used internally */

#if defined BITSPERBYTE
   #define _N_BITSPERBYTE  BITSPERBYTE
#else
   #define _N_BITSPERBYTE  8
#endif

/*
------------------------------------------------------------------------------
Basic Machine Dependent Types:

Define the basic machine dependent type definitions
------------------------------------------------------------------------------
*/

#if defined( N_PLAT_MSW )

#if ! defined(N_RC_INVOKED)

#define N_FAR     far
#define N_NEAR    near
#define N_PTR     far *

#define nint8     signed char
#define nuint8    unsigned char
#define nint16    short
#define nuint16   unsigned short
#define nint32    long
#define nuint32   unsigned long
#define nint      int
#define nuint     unsigned int
#define nbool     unsigned int
#define nreal32   float
#define nreal64   double
#define nreal80   long double
#define nreal     double
#define nhdl      nuint32

/* have to typedef pointer types */

typedef void N_FAR *          nptr;
typedef unsigned char _huge * neptr;

#define npt16     POINT
#define nrect16   RECT

typedef struct tagnpt32
{
   nint32      x;
   nint32      y;
} npt32;

typedef struct tagnpthv16
{
   nint16      h;
   nint16      v;
} npthv16;

typedef struct tagnpthv32
{
   nint32      h;
   nint32      v;
} npthv32;

typedef struct tagnrect32
{
   nint32      left;
   nint32      top;
   nint32      right;
   nint32      bottom;
} nrect32;

typedef struct tagnrectmm16
{
   nint16      xMin;
   nint16      yMin;
   nint16      xMax;
   nint16      yMax;
} nrectmm16;

typedef struct tagnrectmm32
{
   nint32      xMin;
   nint32      yMin;
   nint32      xMax;
   nint32      yMax;
} nrectmm32;

#endif /* ! defined(N_RC_INVOKED) */

#elif defined( N_PLAT_MAC )

#if ! defined(N_RC_INVOKED)

#define N_FAR
#define N_NEAR
#define N_PTR     *

#define nint8     char
#define nuint8    unsigned char
#define nint16    short
#define nuint16   unsigned short
#define nint32    long
#define nuint32   unsigned long
#define nint      int
#define nuint     unsigned int
#define nbool     unsigned char
#define nreal32   float
#define nreal64   double
#define nreal80   extended
#define nreal     extended
#define nhdl      nuint32

/* have to typedef pointer types */

typedef void *             nptr;
typedef unsigned char *    neptr;

#define nrect16   Rect
#define npthv16   Point

typedef struct tagnpt16
{
   nint16      y;
   nint16      x;
} npt16;

typedef struct tagnpt32
{
   nint32      y;
   nint32      x;
} npt32;

typedef struct tagnpthv32
{
   nint32      h;
   nint32      v;
} npthv32;

typedef struct tagnrect32
{
   nint32      top;
   nint32      left;
   nint32      bottom;
   nint32      right;
} nrect32;

typedef struct tagnrectmm16
{
   nint16      yMin;
   nint16      xMin;
   nint16      yMax;
   nint16      xMax;
} nrectmm16;

typedef struct tagnrectmm32
{
   nint32      yMin;
   nint32      xMin;
   nint32      yMax;
   nint32      xMax;
} nrectmm32;

#endif /* ! defined(N_RC_INVOKED) */

#elif defined( N_PLAT_UNIX )

#if ! defined(N_RC_INVOKED)

#define N_FAR
#define N_NEAR
#define N_PTR     *

#define nint8     signed char
/* WARNING:  THE NEXT STATEMENT PRESUMES CHAR IS UNSIGNED!
 */
#ifndef __GNUC__
#define nuint8    unsigned char
#else
#define nuint8    char
#endif
#define nint16    short
#define nuint16   unsigned short
#define nint32    long
#define nuint32   unsigned long
#define nint      long
#define nuint     unsigned long
#define nreal32   float
#define nreal64   double
#define nreal80   long double
#define nreal     double
#define nbool     unsigned short

/* have to typedef pointer types */

typedef void *             nptr;
typedef unsigned char *    neptr;
typedef struct undef *     nhdl;

#define npt16     XPoint

typedef struct tagnpthv16
{
   nint16      h;
   nint16      v;
} npthv16;

typedef struct tagnpthv32
{
   nint32      h;
   nint32      v;
} npthv32;

typedef struct tagnpt32
{
   nint32      x;
   nint32      y;
} npt32;

typedef struct tagnrect16
{
   nint16      left;
   nint16      top;
   nint16      right;
   nint16      bottom;
} nrect16;

typedef struct tagnrectmm16
{
   nint16      xMin;
   nint16      yMin;
   nint16      xMax;
   nint16      yMax;
} nrectmm16;

typedef struct tagnrect32
{
   nint32      left;
   nint32      top;
   nint32      right;
   nint32      bottom;
} nrect32;

typedef struct tagnrectmm32
{
   nint32      xMin;
   nint32      yMin;
   nint32      xMax;
   nint32      yMax;
} nrectmm32;

#endif /* ! defined(N_RC_INVOKED) */

#elif defined( N_PLAT_OS2 )

#if ! defined(N_RC_INVOKED)

#define N_FAR
#define N_NEAR

#if (defined(PT_32BIT) && PT_32BIT) || (defined(__32BIT__) && __32BIT__)
   #define N_PTR     *
#else
   #define N_PTR     * _far
#endif

#define nint8     signed char
#define nuint8    unsigned char
#define nint16    short
#define nuint16   unsigned short
#define nint32    long
#define nuint32   unsigned long
#define nint      int
#define nuint     unsigned int
#define nbool     unsigned short
#define nreal32   float
#define nreal64   double
#define nreal     double
#define nhdl      nuint32

/* have to typedef pointer types */

typedef void N_FAR *          nptr;
typedef unsigned char _huge * neptr;

#define npt16  POINTS
#define npt32  POINTL

typedef struct tagnrect16
{
   nint16      left;
   nint16      bottom;
   nint16      right;
   nint16      top;
} nrect16;

typedef struct tagnpthv16
{
   nint16      h;
   nint16      v;
} npthv16;

typedef struct tagnpthv32
{
   nint32      h;
   nint32      v;
} npthv32;

typedef struct tagnrect32
{
   nint32      left;
   nint32      bottom;
   nint32      right;
   nint32      top;
} nrect32;

typedef struct tagnrectmm16
{
   nint16      xMin;
   nint16      yMax;
   nint16      xMax;
   nint16      yMin;
} nrectmm16;

typedef struct tagnrectmm32
{
   nint32      xMin;
   nint32      yMax;
   nint32      xMax;
   nint32      yMin;
} nrectmm32;

#endif /* ! defined(N_RC_INVOKED) */

#elif defined( N_PLAT_NLM )

#if ! defined(N_RC_INVOKED)

#define N_FAR
#define N_NEAR
#define N_PTR     *

#define nint8     signed char
#define nuint8    unsigned char
#define nint16    short
#define nuint16   unsigned short
#define nint32    long
#define nuint32   unsigned long
#define nint      int
#define nuint     unsigned int
#define nbool     unsigned int
#define nreal32   float
#define nreal64   double
#define nreal80   long double
#define nreal     double
#define nhdl      nuint32

/* have to typedef pointer types */

typedef void N_FAR *          nptr;
typedef unsigned char _huge * neptr;

typedef struct tagnpt16
{
   nint16      x;
   nint16      y;
} npt16;

typedef struct tagnpt32
{
   nint32      x;
   nint32      y;
} npt32;

typedef struct tagnpthv16
{
   nint16      h;
   nint16      v;
} npthv16;

typedef struct tagnpthv32
{
   nint32      h;
   nint32      v;
} npthv32;

typedef struct tagnrect16
{
   nint16      left;
   nint16      top;
   nint16      right;
   nint16      bottom;
} nrect16;

typedef struct tagnrect32
{
   nint32      left;
   nint32      top;
   nint32      right;
   nint32      bottom;
} nrect32;

typedef struct tagnrectmm16
{
   nint16      xMin;
   nint16      yMin;
   nint16      xMax;
   nint16      yMax;
} nrectmm16;

typedef struct tagnrectmm32
{
   nint32      xMin;
   nint32      yMin;
   nint32      xMax;
   nint32      yMax;
} nrectmm32;

#endif /* ! defined(N_RC_INVOKED) */

#elif defined( N_PLAT_DOS )

#if ! defined(N_RC_INVOKED)

#define N_FAR     _far
#define N_NEAR    _near
#define N_PTR     _far *

#define nint8     signed char
#define nuint8    unsigned char
#define nint16    short
#define nuint16   unsigned short
#define nint32    long
#define nuint32   unsigned long
#define nint      int
#define nuint     unsigned int
#define nbool     unsigned int
#define nreal32   float
#define nreal64   double
#define nreal80   long double
#define nreal     double
#define nhdl      nuint32

/* have to typedef pointer types */

typedef void N_FAR *          nptr;
typedef unsigned char _huge * neptr;

typedef struct tagnpt16
{
   nint16      x;
   nint16      y;
} npt16;

typedef struct tagnpt32
{
   nint32      x;
   nint32      y;
} npt32;

typedef struct tagnpthv16
{
   nint16      h;
   nint16      v;
} npthv16;

typedef struct tagnpthv32
{
   nint32      h;
   nint32      v;
} npthv32;

typedef struct tagnrect16
{
   nint16      left;
   nint16      top;
   nint16      right;
   nint16      bottom;
} nrect16;

typedef struct tagnrect32
{
   nint32      left;
   nint32      top;
   nint32      right;
   nint32      bottom;
} nrect32;

typedef struct tagnrectmm16
{
   nint16      xMin;
   nint16      yMin;
   nint16      xMax;
   nint16      yMax;
} nrectmm16;

typedef struct tagnrectmm32
{
   nint32      xMin;
   nint32      yMin;
   nint32      xMax;
   nint32      yMax;
} nrectmm32;

#endif /* ! defined(N_RC_INVOKED) */

#else
   HELP SYSTEM_ARCHITECTURE_IS_UNDEFINED;
#endif    /* MSW or MAC or UNIX or OS2 undefined */

/*
------------------------------------------------------------------------------
Basic Derived Types:

Define the basic derived type definitions
------------------------------------------------------------------------------
*/

#if ! defined(N_RC_INVOKED)

#define nid                nuint32
#define nflag8             nuint8
#define nflag16            nuint16
#define nflag32            nuint32
#define nbool8             nuint8
#define nbool16            nuint16
#define nbool32            nuint32
#define nfixed             nint32
#define nstr8              nuint8
#define nstr16             nuint16
#if defined(N_USE_STR_16)
#define nstr               nstr16
#else
#define nstr               nstr8
#endif
#define nchar              nstr
#define nchar8             nstr8
#define nchar16            nstr16
#define nwchar             nchar16
#define nparam             nuint32
typedef void N_FAR *       npproc;

/*
   Pointers to scalars.
*/

typedef nint8     N_FAR *  pnint8;
typedef nuint8    N_FAR *  pnuint8;
typedef nint16    N_FAR *  pnint16;
typedef nuint16   N_FAR *  pnuint16;
typedef nint32    N_FAR *  pnint32;
typedef nuint32   N_FAR *  pnuint32;
#if defined( nint64 )
typedef nint64    N_FAR *  pnint64;
typedef nuint64   N_FAR *  pnuint64;
#endif
typedef nint      N_FAR *  pnint;
typedef nuint     N_FAR *  pnuint;
typedef nflag8    N_FAR *  pnflag8;
typedef nflag16   N_FAR *  pnflag16;
typedef nflag32   N_FAR *  pnflag32;
typedef nbool     N_FAR *  pnbool;
typedef nbool8    N_FAR *  pnbool8;
typedef nbool16   N_FAR *  pnbool16;
typedef nbool32   N_FAR *  pnbool32;
typedef nfixed    N_FAR *  pnfixed;
typedef nstr      N_FAR *  pnstr;
typedef pnstr     N_FAR *  ppnstr;
typedef nstr16    N_FAR *  pnstr16;
typedef pnstr16   N_FAR *  ppnstr16;
typedef nstr8     N_FAR *  pnstr8;
typedef pnstr8    N_FAR *  ppnstr8;
typedef nchar     N_FAR *  pnchar;
typedef pnchar    N_FAR *  ppnchar;
typedef nchar16   N_FAR *  pnchar16;
typedef pnchar16  N_FAR *  ppnchar16;
typedef nchar8    N_FAR *  pnchar8;
typedef pnchar8   N_FAR *  ppnchar8;
typedef nwchar    N_FAR *  pnwchar;
typedef pnwchar   N_FAR *  ppnwchar;
typedef nreal     N_FAR *  pnreal;
typedef nreal32   N_FAR *  pnreal32;
typedef nreal64   N_FAR *  pnreal64;
#if defined( nreal80 )
typedef nreal80   N_FAR *  pnreal80;
#endif
typedef nid       N_FAR *  pnid;
typedef nhdl      N_FAR *  pnhdl;
typedef nparam    N_FAR *  pnparam;
typedef nptr      N_FAR *  pnptr;
typedef neptr     N_FAR *  pneptr;

/*
   Pointers to structures.
*/

typedef nrect16   N_FAR *  pnrect16;
typedef nrect32   N_FAR *  pnrect32;
typedef nrectmm16 N_FAR *  pnrectmm16;
typedef nrectmm32 N_FAR *  pnrectmm32;
typedef npt16     N_FAR *  pnpt16;
typedef npt32     N_FAR *  pnpt32;
typedef npproc    N_FAR *  pnpproc;

#endif   /* ! defined(N_RC_INVOKED) */

/*
------------------------------------------------------------------------------
Macros:

------------------------------------------------------------------------------
*/

#if ! defined(N_RC_INVOKED)

#if defined( N_PLAT_MSW )

#if defined( __cplusplus )

/* Special 'extern' for C++ to avoid name mangling... */

#define _N_EXTERN       extern "C"

#else
#define _N_EXTERN       extern
#endif /* __cplusplus */

#define N_REG1          register
#define N_REG2          register
#define N_REG3          register
#define N_REG4          register

#define N_STATIC_VAR    static
#define N_INTERN_VAR    static
#define N_GLOBAL_VAR
#define N_EXTERN_VAR    extern

#define N_UNUSED_VAR( x ) x = x

#define N_INTERN_FUNC( retType )          static retType N_NEAR
#define N_INTERN_FUNC_C( retType )        static retType N_NEAR
#define N_INTERN_FUNC_PAS( retType )      static retType N_NEAR PASCAL
#define N_GLOBAL_FUNC( retType )          retType
#define N_EXTERN_FUNC( retType )          _N_EXTERN retType
#define N_GLOBAL_FUNC_C( retType )        retType
#define N_EXTERN_FUNC_C( retType )        _N_EXTERN retType
#define N_GLOBAL_FUNC_PAS( retType )      retType PASCAL
#define N_EXTERN_FUNC_PAS( retType )      _N_EXTERN retType PASCAL
#define N_GLOBAL_LIBRARY( retType )       retType N_FAR PASCAL
#define N_EXTERN_LIBRARY( retType )       _N_EXTERN retType N_FAR PASCAL
#define N_GLOBAL_LIBRARY_C( retType )     retType N_FAR __cdecl
#define N_EXTERN_LIBRARY_C( retType )     _N_EXTERN retType N_FAR __cdecl
#define N_GLOBAL_LIBRARY_PAS( retType )   retType N_FAR PASCAL
#define N_EXTERN_LIBRARY_PAS( retType )   _N_EXTERN retType N_FAR PASCAL
#define N_GLOBAL_CALLBACK( retType )      retType N_FAR PASCAL _export
#define N_EXTERN_CALLBACK( retType )      _N_EXTERN retType N_FAR PASCAL \
                                          _export
#define N_GLOBAL_CALLBACK_C( retType )    retType N_FAR __cdecl _export
#define N_EXTERN_CALLBACK_C( retType )    _N_EXTERN retType N_FAR __cdecl \
                                          _export
#define N_GLOBAL_CALLBACK_PAS( retType )  retType N_FAR PASCAL _export
#define N_EXTERN_CALLBACK_PAS( retType )  _N_EXTERN retType N_FAR PASCAL \
                                          _export

#define N_TYPEDEF_FUNC( retType, typeName ) \
            typedef retType (*typeName)

#define N_TYPEDEF_FUNC_C( retType, typeName ) \
            typedef retType (*typeName)

#define N_TYPEDEF_FUNC_PAS( retType, typeName ) \
            typedef retType (PASCAL *typeName)

#define N_TYPEDEF_LIBRARY( retType, typeName ) \
            typedef retType (N_FAR PASCAL *typeName)

#define N_TYPEDEF_LIBRARY_C( retType, typeName ) \
            typedef retType (N_FAR __cdecl *typeName)

#define N_TYPEDEF_LIBRARY_PAS( retType, typeName ) \
            typedef retType (N_FAR PASCAL *typeName)

#define N_TYPEDEF_CALLBACK( retType, typeName ) \
            typedef retType (N_FAR PASCAL _export *typeName)

#define N_TYPEDEF_CALLBACK_C( retType, typeName ) \
            typedef retType (N_FAR __cdecl _export *typeName)

#define N_TYPEDEF_CALLBACK_PAS( retType, typeName ) \
            typedef retType (N_FAR PASCAL _export *typeName)

#define N_TYPEDEF_INTERN_FUNC( retType, typeName ) \
            typedef retType (N_NEAR *typeName)

#define N_TYPEDEF_INTERN_FUNC_C( retType, typeName ) \
            typedef retType (N_NEAR *typeName)

#define N_TYPEDEF_INTERN_FUNC_PAS( retType, typeName ) \
            typedef retType (N_NEAR PASCAL *typeName)

#elif defined( N_PLAT_MAC )

#define N_REG1             register
#define N_REG2             register
#define N_REG3             register
#define N_REG4             register

#define N_STATIC_VAR       static
#define N_INTERN_VAR       static
#define N_GLOBAL_VAR
#define N_EXTERN_VAR       extern

#if applec
   #define N_UNUSED_VAR( x )     #pragma unused( x )
#else
   #define N_UNUSED_VAR( x )
#endif

#define N_INTERN_FUNC( retType )         static retType
#define N_INTERN_FUNC_C( retType )       static retType
#define N_INTERN_FUNC_PAS( retType )     static pascal retType

#define N_GLOBAL_FUNC( retType )         retType
#define N_EXTERN_FUNC( retType )         extern retType
#define N_GLOBAL_FUNC_C( retType )       retType
#define N_EXTERN_FUNC_C( retType )       extern retType
#define N_GLOBAL_FUNC_PAS( retType )     pascal retType
#define N_EXTERN_FUNC_PAS( retType )     extern pascal retType

#define N_GLOBAL_LIBRARY( retType )      pascal retType
#define N_EXTERN_LIBRARY( retType )      extern pascal retType
#define N_GLOBAL_LIBRARY_C( retType )    retType
#define N_EXTERN_LIBRARY_C( retType )    extern retType
#define N_GLOBAL_LIBRARY_PAS( retType )  pascal retType
#define N_EXTERN_LIBRARY_PAS( retType )  extern pascal retType

#define N_GLOBAL_CALLBACK( retType )     pascal retType
#define N_EXTERN_CALLBACK( retType )     extern pascal retType
#define N_GLOBAL_CALLBACK_C( retType )   retType
#define N_EXTERN_CALLBACK_C( retType )   extern retType
#define N_GLOBAL_CALLBACK_PAS( retType ) pascal retType
#define N_EXTERN_CALLBACK_PAS( retType ) extern pascal retType

#define N_TYPEDEF_FUNC( retType, typeName ) \
               typedef retType (*typeName)
#define N_TYPEDEF_FUNC_C( retType, typeName ) \
               typedef retType (*typeName)
#define N_TYPEDEF_FUNC_PAS( retType, typeName ) \
               typedef pascal retType (*typeName)
#define N_TYPEDEF_LIBRARY( retType, typeName ) \
               typedef pascal retType (*typeName)
#define N_TYPEDEF_LIBRARY_C( retType, typeName ) \
               typedef retType (*typeName)
#define N_TYPEDEF_LIBRARY_PAS( retType, typeName ) \
               typedef pascal retType (*typeName)
#define N_TYPEDEF_CALLBACK( retType, typeName ) \
               typedef pascal retType (*typeName)
#define N_TYPEDEF_CALLBACK_C( retType, typeName ) \
               typedef retType (*typeName)
#define N_TYPEDEF_CALLBACK_PAS( retType, typeName ) \
               typedef pascal retType (*typeName)
#define N_TYPEDEF_INTERN_FUNC( retType, typeName ) \
               typedef retType (*typeName)
#define N_TYPEDEF_INTERN_FUNC_C( retType, typeName ) \
               typedef retType (*typeName)
#define N_TYPEDEF_INTERN_FUNC_PAS( retType, typeName ) \
               typedef pascal retType (*typeName)

#elif defined( N_PLAT_UNIX )

#if defined(__cplusplus)
#define _N_EXTERN          extern "C"
#else
#define _N_EXTERN          extern
#endif /* __cplusplus */

#define N_REG1          register
#define N_REG2          register
#define N_REG3          register
#define N_REG4          register

#define N_STATIC_VAR    static
#define N_INTERN_VAR    static
#define N_GLOBAL_VAR
#define N_EXTERN_VAR    extern

#define N_UNUSED_VAR( x ) x = x

#define N_INTERN_FUNC( retType )          static retType
#define N_INTERN_FUNC_C( retType )        static retType
#define N_INTERN_FUNC_PAS( retType )      static retType
#define N_GLOBAL_FUNC( retType )          retType
#define N_EXTERN_FUNC( retType )          _N_EXTERN retType
#define N_GLOBAL_FUNC_C( retType )        retType
#define N_EXTERN_FUNC_C( retType )        _N_EXTERN retType
#define N_GLOBAL_FUNC_PAS( retType )      retType
#define N_EXTERN_FUNC_PAS( retType )      _N_EXTERN retType
#define N_GLOBAL_LIBRARY( retType )       retType
#define N_EXTERN_LIBRARY( retType )       _N_EXTERN retType
#define N_GLOBAL_LIBRARY_C( retType )     retType
#define N_EXTERN_LIBRARY_C( retType )     _N_EXTERN retType
#define N_GLOBAL_LIBRARY_PAS( retType )   retType
#define N_EXTERN_LIBRARY_PAS( retType )   _N_EXTERN retType
#define N_GLOBAL_CALLBACK( retType )      retType
#define N_EXTERN_CALLBACK( retType )      _N_EXTERN retType
#define N_GLOBAL_CALLBACK_C( retType )    retType
#define N_EXTERN_CALLBACK_C( retType )    _N_EXTERN retType
#define N_GLOBAL_CALLBACK_PAS( retType )  retType
#define N_EXTERN_CALLBACK_PAS( retType )  _N_EXTERN retType

#define N_TYPEDEF_FUNC( retType, typeName ) \
               typedef retType (*typeName)
#define N_TYPEDEF_FUNC_C( retType, typeName ) \
               typedef retType (*typeName)
#define N_TYPEDEF_FUNC_PAS( retType, typeName ) \
               typedef retType (*typeName)
#define N_TYPEDEF_LIBRARY( retType, typeName ) \
               typedef retType (*typeName)
#define N_TYPEDEF_LIBRARY_C( retType, typeName ) \
               typedef retType (*typeName)
#define N_TYPEDEF_LIBRARY_PAS( retType, typeName ) \
               typedef retType (*typeName)
#define N_TYPEDEF_CALLBACK( retType, typeName ) \
               typedef retType (*typeName)
#define N_TYPEDEF_CALLBACK_C( retType, typeName ) \
               typedef retType (*typeName)
#define N_TYPEDEF_CALLBACK_PAS( retType, typeName ) \
               typedef retType (*typeName)
#define N_TYPEDEF_INTERN_FUNC( retType, typeName ) \
               typedef retType (*typeName)
#define N_TYPEDEF_INTERN_FUNC_C( retType, typeName ) \
               typedef retType (*typeName)
#define N_TYPEDEF_INTERN_FUNC_PAS( retType, typeName ) \
               typedef retType (*typeName)

#elif defined( N_PLAT_OS2)

#define N_PRAGMA_ONCE

#if defined( __cplusplus )

/* Special 'extern' for C++ to avoid name mangling... */

#define _N_EXTERN       extern "C"

#else
#define _N_EXTERN       extern
#endif /* __cplusplus */

#define _N_PASCAL       _Pascal

#define N_REG1          register
#define N_REG2          register
#define N_REG3          register
#define N_REG4          register

#define N_STATIC_VAR    static
#define N_INTERN_VAR    static
#define N_GLOBAL_VAR
#define N_EXTERN_VAR    extern

#define N_UNUSED_VAR( x ) x = x

#define N_INTERN_FUNC( retType )          static retType
#define N_INTERN_FUNC_C( retType )        static retType
#define N_INTERN_FUNC_PAS( retType )      static retType  _N_PASCAL
#define N_GLOBAL_FUNC( retType )          retType
#define N_EXTERN_FUNC( retType )          _N_EXTERN retType
#define N_GLOBAL_FUNC_C( retType )        retType
#define N_EXTERN_FUNC_C( retType )        _N_EXTERN retType
#define N_GLOBAL_FUNC_PAS( retType )      retType _N_PASCAL
#define N_EXTERN_FUNC_PAS( retType )      _N_EXTERN retType  _N_PASCAL
#define N_GLOBAL_LIBRARY( retType )       retType  _N_PASCAL
#define N_EXTERN_LIBRARY( retType )       _N_EXTERN retType  _N_PASCAL
#define N_GLOBAL_LIBRARY_C( retType )     retType
#define N_EXTERN_LIBRARY_C( retType )     _N_EXTERN retType
#define N_GLOBAL_LIBRARY_PAS( retType )   retType  _N_PASCAL
#define N_EXTERN_LIBRARY_PAS( retType )   _N_EXTERN retType  _N_PASCAL
#define N_GLOBAL_CALLBACK( retType )      retType  _N_PASCAL  _export
#define N_EXTERN_CALLBACK( retType )      _N_EXTERN retType  _N_PASCAL \
                                          _export
#define N_GLOBAL_CALLBACK_NATIVE( retType )     retType  EXPENTRY _export
#define N_EXTERN_CALLBACK_NATIVE( retType )     _N_EXTERN retType  EXPENTRY \
                                          _export
#define N_GLOBAL_CALLBACK_C( retType )    retType  _export
#define N_EXTERN_CALLBACK_C( retType )    _N_EXTERN retType  _export
#define N_GLOBAL_CALLBACK_PAS( retType )  retType  _N_PASCAL _export
#define N_EXTERN_CALLBACK_PAS( retType )  _N_EXTERN retType  _N_PASCAL \
                                          _export

#define N_TYPEDEF_FUNC( retType, typeName ) \
            typedef retType (*typeName)

#define N_TYPEDEF_FUNC_C( retType, typeName ) \
            typedef retType (*typeName)

#define N_TYPEDEF_FUNC_PAS( retType, typeName ) \
            typedef retType (* _N_PASCAL typeName)

#define N_TYPEDEF_LIBRARY( retType, typeName ) \
            typedef retType ( * _N_PASCAL typeName)

#define N_TYPEDEF_LIBRARY_C( retType, typeName ) \
            typedef retType ( *typeName)

#define N_TYPEDEF_LIBRARY_PAS( retType, typeName ) \
            typedef retType (*  _N_PASCAL typeName)

#define N_TYPEDEF_CALLBACK( retType, typeName ) \
            typedef  retType (  _export * _N_PASCAL typeName)

#define N_TYPEDEF_CALLBACK_NATIVE( retType, typeName ) \
            typedef  retType (  _export * EXPENTRY  typeName)

#define N_TYPEDEF_CALLBACK_C( retType, typeName ) \
            typedef retType ( _export *typeName)

#define N_TYPEDEF_CALLBACK_PAS( retType, typeName ) \
            typedef retType ( _export  *  _N_PASCAL typeName)

#define N_TYPEDEF_INTERN_FUNC( retType, typeName ) \
            typedef retType ( *typeName)

#define N_TYPEDEF_INTERN_FUNC_C( retType, typeName ) \
            typedef retType ( *typeName)

#define N_TYPEDEF_INTERN_FUNC_PAS( retType, typeName ) \
            typedef retType (* _N_PASCAL typeName)

#elif defined( N_PLAT_DOS )
#if !defined(PASCAL)
#define PASCAL _pascal
#endif

#if defined( __cplusplus )

/* Special 'extern' for C++ to avoid name mangling... */

#define _N_EXTERN       extern "C"

#else
#define _N_EXTERN       extern
#endif /* __cplusplus */

#define N_REG1          register
#define N_REG2          register
#define N_REG3          register
#define N_REG4          register

#define N_STATIC_VAR    static
#define N_INTERN_VAR    static
#define N_GLOBAL_VAR
#define N_EXTERN_VAR    extern

#define N_UNUSED_VAR( x ) x = x

#define N_INTERN_FUNC( retType )          static retType N_NEAR
#define N_INTERN_FUNC_C( retType )        static retType N_NEAR
#define N_INTERN_FUNC_PAS( retType )      static retType N_NEAR PASCAL
#define N_GLOBAL_FUNC( retType )          retType
#define N_EXTERN_FUNC( retType )          _N_EXTERN retType
#define N_GLOBAL_FUNC_C( retType )        retType
#define N_EXTERN_FUNC_C( retType )        _N_EXTERN retType
#define N_GLOBAL_FUNC_PAS( retType )      retType PASCAL
#define N_EXTERN_FUNC_PAS( retType )      _N_EXTERN retType PASCAL
#define N_GLOBAL_LIBRARY( retType )       retType N_FAR PASCAL
#define N_EXTERN_LIBRARY( retType )       _N_EXTERN retType N_FAR PASCAL
#define N_GLOBAL_LIBRARY_C( retType )     retType N_FAR
#define N_EXTERN_LIBRARY_C( retType )     _N_EXTERN retType N_FAR
#define N_GLOBAL_LIBRARY_PAS( retType )   retType N_FAR PASCAL
#define N_EXTERN_LIBRARY_PAS( retType )   _N_EXTERN retType N_FAR PASCAL
#define N_GLOBAL_CALLBACK( retType )      retType N_FAR PASCAL
#define N_EXTERN_CALLBACK( retType )      _N_EXTERN retType N_FAR PASCAL
#define N_GLOBAL_CALLBACK_C( retType )    retType N_FAR
#define N_EXTERN_CALLBACK_C( retType )    _N_EXTERN retType N_FAR
#define N_GLOBAL_CALLBACK_PAS( retType )  retType N_FAR PASCAL
#define N_EXTERN_CALLBACK_PAS( retType )  _N_EXTERN retType N_FAR PASCAL

#define N_TYPEDEF_FUNC( retType, typeName ) \
            typedef retType (*typeName)

#define N_TYPEDEF_FUNC_C( retType, typeName ) \
            typedef retType (*typeName)

#define N_TYPEDEF_FUNC_PAS( retType, typeName ) \
            typedef retType (PASCAL *typeName)

#define N_TYPEDEF_LIBRARY( retType, typeName ) \
            typedef retType (N_FAR PASCAL *typeName)

#define N_TYPEDEF_LIBRARY_C( retType, typeName ) \
            typedef retType (N_FAR *typeName)

#define N_TYPEDEF_LIBRARY_PAS( retType, typeName ) \
            typedef retType (N_FAR PASCAL *typeName)

#define N_TYPEDEF_CALLBACK( retType, typeName ) \
            typedef retType (N_FAR PASCAL _export *typeName)

#define N_TYPEDEF_CALLBACK_C( retType, typeName ) \
            typedef retType (N_FAR _export *typeName)

#define N_TYPEDEF_CALLBACK_PAS( retType, typeName ) \
            typedef retType (N_FAR PASCAL _export *typeName)

#define N_TYPEDEF_INTERN_FUNC( retType, typeName ) \
            typedef retType (N_NEAR *typeName)

#define N_TYPEDEF_INTERN_FUNC_C( retType, typeName ) \
            typedef retType (N_NEAR *typeName)

#define N_TYPEDEF_INTERN_FUNC_PAS( retType, typeName ) \
            typedef retType (N_NEAR PASCAL *typeName)

#endif /* N_PLAT_DOS */

/*
   Untyped logical macros.
*/

#define NMin(a,b)          ((a) < (b) ? (a) : (b))
#define NMax(a,b)          ((a) > (b) ? (a) : (b))

#endif /* ! defined( N_RC_INVOKED ) */

/*
   Integer construction macros.
*/

#if defined( N_RC_INVOKED )

/* NMakePtrParam isn't defined since it can't be used in a resource file */

#define NMake32(a,b)       ((a) | ((b) << 16))
#define NGetLo16(a32)      ((a32) & 0xFFFF)
#define NGetHi16(a32)      (((a32) >> 16) & 0xFFFF)
#define NMake16(a,b)       ((a) | ((b) << 8))
#define NGetLo8(a16)       ((a16) & 0xFF)
#define NGetHi8(a16)       (((a16) >> 8) & 0xFF)

#else /* not N_RC_INVOKED */

/* _N_BITS is currently only used internally */

#if defined( BITS )
   #define _N_BITS         BITS
#else
   #define _N_BITS(type)   (_N_BITSPERBYTE * (int)sizeof(type))
#endif

#define NMakePtrParam(p)   ((nuint32) ((nptr) (p)))

/* #define NMake32(a,b)       ((a) | ((b) << 16)) */

#define NMake32(a,b)       ((nuint32) (((nuint16)(a)) | (((nuint32)(nuint16)(b)) << (_N_BITS(nuint16)))))
#define NGetLo16(a32)      ((nuint16)a32)
#define NGetHi16(a32)      ((nuint16)((nuint32)(a32) >> _N_BITS(nuint16)))
#define NMake16(a,b)       ((nuint16) (((nuint8)(a)) | \
                           (((nuint16)((nuint8)(b))) << _N_BITS(nuint8))))
#define NGetLo8(a16)       ((nuint8)(a16))
#define NGetHi8(a16)       ((nuint8)((nuint16)(a16) >> _N_BITS(nuint8)))

#endif /* not N_RC_INVOKED */

#define NMakePair32        NMake32
#define NGetFirst16(a)     ((nint16) NGetLo16(a))
#define NGetSecond16(a)    ((nint16) NGetHi16(a))

#define NMakePair16        NMake16
#define NGetFirst8(a)      ((nint8) NGetLo8(a))
#define NGetSecond8(a)     ((nint8) NGetHi8(a))

#define NMakeFixed32(i,f)  NMake32(f,i)
#define NGetFixedInt16(a)  ((nint16) NGetHi16(a))
#define NGetFixedFrac16(a) ((nint16) NGetLo16(a))

/*
   Integer representation macros.
*/

#if defined( N_PLAT_MSW )

   /* MSW only runs on big-endian machines, including NT Risc machines */

   #undef  N_INT_ENDIAN_HI_LO

   /* TBD: strict alignment does apply for some NT Risc machines */

   #undef  N_INT_STRICT_ALIGNMENT

#elif defined( N_PLAT_MAC )

   /* Even the Power PC uses big-endian */

   #define N_INT_ENDIAN_HI_LO

   /* Strict alignment applies except when compiling explicitly for
      the 68020 and above */

   #if defined( mc68020 )
      #undef  N_INT_STRICT_ALIGNMENT
   #else
      #define N_INT_STRICT_ALIGNMENT
   #endif

#elif defined( N_PLAT_UNIX )

   #if defined(BYTE_ORDER) && defined(BIG_ENDIAN)
       #if (BYTE_ORDER == BIG_ENDIAN)
           #define N_INT_ENDIAN_HI_LO
       #endif
   #elif defined(vax) || defined(ns32000) || defined(sun386) || \
           defined(MIPSEL) || defined(BIT_ZERO_ON_RIGHT)
       /* do nothing (little endian) */
   #elif defined(sel) || defined(pyr) || defined(mc68000) || \
           defined(sparc) || defined(is68k) || defined(tahoe) || \
           defined(ibm032) || defined(ibm370) || defined(MIPSEB) || \
           defined(__hpux) || defined (BIT_ZERO_ON_LEFT)
       #define N_INT_ENDIAN_HI_LO
   #endif

   /* For now, we assume strict alignment for all Unix platforms since it
      is the worst case (safe); we could optimize for certain platforms
      in the future */

   #define N_INT_STRICT_ALIGNMENT

#elif defined( N_PLAT_OS2)

   /* We only support OS2 on Intel machines currently */

   #undef  N_INT_ENDIAN_HI_LO
   #undef  N_INT_STRICT_ALIGNMENT

#elif defined( N_PLAT_DOS )

   /* We only support DOS on Intel machines currently */

   #undef  N_INT_ENDIAN_HI_LO
   #undef  N_INT_STRICT_ALIGNMENT

#endif

/*
// NSwapXxYyZz group
*/

#if defined( N_INT_STRICT_ALIGNMENT )
#define NGet32( x )  ((nuint32) ( \
         (((nuint32)(((pnuint8)(&(x)))[3])) << 24) | \
         (((nuint32)(((pnuint8)(&(x)))[2])) << 16) | \
         (((nuint32)(((pnuint8)(&(x)))[1])) << 8)  | \
         (((nuint32)(((pnuint8)(&(x)))[0]))     )  ))

#define NGet16( x )  ((nuint16) ( \
         (((nuint16)(((pnuint8)(&(x)))[1])) << 8)  | \
         (((nuint16)(((pnuint8)(&(x)))[0]))     )  ))

#define NSwap32( x ) ((nuint32) ( \
         (((nuint32)(((pnuint8)(&(x)))[0])) << 24) | \
         (((nuint32)(((pnuint8)(&(x)))[1])) << 16) | \
         (((nuint32)(((pnuint8)(&(x)))[2])) << 8)  | \
         (((nuint32)(((pnuint8)(&(x)))[3]))     )  ))

#define NSwap16( x ) ((nuint16) ( \
         (((nuint16)(((pnuint8)(&(x)))[0])) << 8)  | \
         (((nuint16)(((pnuint8)(&(x)))[1]))     )  ))
#else
#define NGet32( x )  (*((pnuint32) (&(x))))

#define NGet16( x )  (*((pnuint16) (&(x))))

#define NSwap32( x ) ((nuint32) ( \
         ((*((pnuint32)&(x)) & 0x000000FFL) << 24) | \
         ((*((pnuint32)&(x)) & 0x0000FF00L) <<  8) | \
         ((*((pnuint32)&(x)) & 0x00FF0000L) >>  8) | \
         ((*((pnuint32)&(x)) & 0xFF000000L) >> 24) ))

#define NSwap16( x ) ((nuint16) ( \
         ((*((pnuint16)&(x)) & 0x00FF) << 8) | \
         ((*((pnuint16)&(x)) & 0xFF00) >> 8) ))
#endif



#if defined( N_INT_ENDIAN_HI_LO )

   #define NSwapHiLo32( x )      NGet32( x )
   #define NSwapHiLo16( x )      NGet16( x )
   #define NSwapLoHi32( x )      NSwap32( x )
   #define NSwapLoHi16( x )      NSwap16( x )

#else

   #define NSwapLoHi32( x )      NGet32( x )
   #define NSwapLoHi16( x )      NGet16( x )
   #define NSwapHiLo32( x )      NSwap32( x )
   #define NSwapHiLo16( x )      NSwap16( x )

#endif

/*
// NCopyXxYyZz group
*/

#define NCopySwap32( pDest, pSrc )  \
      (((pnuint8) (pDest))[0] = ((pnuint8) (pSrc))[3], \
       ((pnuint8) (pDest))[1] = ((pnuint8) (pSrc))[2], \
       ((pnuint8) (pDest))[2] = ((pnuint8) (pSrc))[1], \
       ((pnuint8) (pDest))[3] = ((pnuint8) (pSrc))[0])

#define NCopySwap16( pDest, pSrc )  \
      (((pnuint8) (pDest))[0] = ((pnuint8) (pSrc))[1], \
       ((pnuint8) (pDest))[1] = ((pnuint8) (pSrc))[0])

#if defined( N_INT_STRICT_ALIGNMENT )

   #define NCopy32( pDest, pSrc )   \
         (((pnuint8) (pDest))[0] = ((pnuint8) (pSrc))[0], \
          ((pnuint8) (pDest))[1] = ((pnuint8) (pSrc))[1], \
          ((pnuint8) (pDest))[2] = ((pnuint8) (pSrc))[2], \
          ((pnuint8) (pDest))[3] = ((pnuint8) (pSrc))[3])

   #define NCopy16( pDest, pSrc )   \
         (((pnuint8) (pDest))[0] = ((pnuint8) (pSrc))[0], \
          ((pnuint8) (pDest))[1] = ((pnuint8) (pSrc))[1])
#else

   #define NCopy32( pDest, pSrc )   \
         (*((pnuint32) (pDest)) = *((pnuint32) (pSrc)))

   #define NCopy16( pDest, pSrc )   \
         (*((pnuint16) (pDest)) = *((pnuint16) (pSrc)))

#endif

#if defined( N_INT_ENDIAN_HI_LO )

   #define NCopyHiLo32        NCopy32
   #define NCopyHiLo16        NCopy16
   #define NCopyLoHi32        NCopySwap32
   #define NCopyLoHi16        NCopySwap16

#else

   #define NCopyHiLo32        NCopySwap32
   #define NCopyHiLo16        NCopySwap16
   #define NCopyLoHi32        NCopy32
   #define NCopyLoHi16        NCopy16

#endif

/* #define NPad32( s )           (((s) + 3) & ~3) */
#define NPad32( s )           ((s + 3) & ~3)
#define NPad16( s )           (((s) + 1) & ~1)

#define NAlign32(s)  *(s) = (void N_FAR *) (((nuint32)*(s) + 3) & ~3);

#define NCopyRect32( prtlTo, prtFrom ) \
   ((prtlTo)->left = (nint32) (prtFrom)->left, \
   (prtlTo)->top = (nint32) (prtFrom)->top, \
   (prtlTo)->right = (nint32) (prtFrom)->right, \
   (prtlTo)->bottom = (nint32) (prtFrom)->bottom)

#define NCopyRect16( prtTo, prtlFrom ) \
   ((prtTo)->left = (nint16) (prtlFrom)->left, \
   (prtTo)->top = (nint16) (prtlFrom)->top, \
   (prtTo)->right = (nint16) (prtlFrom)->right, \
   (prtTo)->bottom = (nint16) (prtlFrom)->bottom)

#define NCopyPt32( pptlTo, pptFrom ) \
   ((pptlTo)->x = (nint32) (pptFrom)->x, \
   (pptlTo)->y = (nint32) (pptFrom)->y)

#define NCopyPt16( pptTo, pptlFrom ) \
   ((pptTo)->x = (nint16) (pptlFrom)->x, \
   (pptTo)->y = (nint16) (pptlFrom)->y)

/*
   Character and String macros.
*/

#if defined(N_USE_STR_16)
   #define NText(a)     L a
#else
   #define NText(a)     a
#endif

#if defined( N_PLAT_MSW ) || defined( N_PLAT_OS2 ) || defined( N_PLAT_DOS )
   #define N_NEWLINE    NText( "\r\n" )
#else
   #define N_NEWLINE    NText( "\n" )
#endif

/*
   Historical macro synonyms.
*/

#if ! defined( N_INC_NO_OLD_MACROS )
   #if !defined( FAR )
      #define FAR       N_FAR
   #endif
   #if !defined( NEAR )
      #define NEAR      N_NEAR
   #endif
   #if !defined( MIN )
      #define MIN       NMin
   #endif
   #if !defined( MAX )
      #define MAX       NMax
   #endif
   #if !defined( MAKELONG )
      #define MAKELONG  NMake32
   #endif
   #if !defined( HIWORD )
      #define HIWORD    NGetHi16
   #endif
   #if !defined( LOWORD )
      #define LOWORD    NGetLo16
   #endif
   #if !defined( MAKEWORD )
      #define MAKEWORD  NMake16
   #endif
   #if !defined( HIBYTE )
      #define HIBYTE    NGetHi8
   #endif
   #if !defined( LOBYTE )
      #define LOBYTE    NGetLo8
   #endif
#endif

#endif /* ! defined( NTYPES_H ) */

#endif
